unit MapiW;

{< Mapi interface for Unicode version MAPISendMailW

  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Feb 2016)
  @lastmod(Feb 2016)
}

interface

uses Windows, Mapi;

type
  PMapiRecipDescW = ^TMapiRecipDescW;
  {$EXTERNALSYM MapiRecipDescW}
  MapiRecipDescW = packed record
    ulReserved: Cardinal;       { Reserved for future use                  }
    ulRecipClass: Cardinal;     { Recipient class                          }
                                { MAPI_TO, MAPI_CC, MAPI_BCC, MAPI_ORIG    }
    lpszName: PWideChar;        { Recipient name                           }
    lpszAddress: PWideChar;     { Recipient address (optional)             }
    ulEIDSize: Cardinal;        { Count in bytes of size of pEntryID       }
    lpEntryID: Pointer;         { System-specific recipient reference      }
  end;
  TMapiRecipDescW = MapiRecipDescW;

  PMapiFileDescW = ^TMapiFileDescW;
  {$EXTERNALSYM MapiFileDescW}
  MapiFileDescW = packed record
    ulReserved: Cardinal;        { Reserved for future use (must be 0)     }
    flFlags: Cardinal;           { Flags                                   }
    nPosition: Cardinal;         { character in text to be replaced by attachment }
    lpszPathName: PWideChar;     { Full path name of attachment file       }
    lpszFileName: PWideChar;     { Original file name (optional)           }
    lpFileType: Pointer;         { Attachment file type (can be lpMapiFileTagExt) }
  end;
  TMapiFileDescW = MapiFileDescW;

  PMapiMessageW = ^TMapiMessageW;
  {$EXTERNALSYM MapiMessageW}
  MapiMessageW = packed record
    ulReserved: Cardinal;         { Reserved for future use (M.B. 0)       }
    lpszSubject: PWideChar;       { Message Subject                        }
    lpszNoteText: PWideChar;      { Message Text                           }
    lpszMessageType: PWideChar;   { Message Class                          }
    lpszDateReceived: PWideChar;  { in YYYY/MM/DD HH:MM format             }
    lpszConversationID: PWideChar;{ conversation thread ID                 }
    flFlags: FLAGS;               { unread,return receipt                  }
    lpOriginator: PMapiRecipDescW; { Originator descriptor                  }
    nRecipCount: Cardinal;        { Number of recipients                   }
    lpRecips: PMapiRecipDescW;     { Recipient descriptors                  }
    nFileCount: Cardinal;         { # of file attachments                  }
    lpFiles: PMapiFileDescW;       { Attachment descriptors                 }
  end;
  TMapiMessageW = MapiMessageW;

  PFNMapiSendMailW = ^TFNMapiSendMailW;
  TFNMapiSendMailW = function(lhSession: LHANDLE; ulUIParam: Cardinal;
    var lpMessageW: TMapiMessageW; flFlags: FLAGS;
    ulReserved: Cardinal): Cardinal stdcall;

function CheckMapiLibrary: boolean;
function CheckMapiUnicode: boolean;

function MapiSendMailW(lhSession: LHANDLE; ulUIParam: Cardinal;
  var lpMessageW: TMapiMessageW; flFlags: FLAGS; ulReserved: Cardinal): Cardinal;


implementation

var
  MAPIModuleW: HModule = 0;
  MAPICheckedW: Boolean = False;
  SendMailW: TFNMapiSendMailW = nil;

procedure InitMapi;
var
  OSVersionInfo: TOSVersionInfo;
  hkWMS: HKEY;
  MAPIValueSize: Longint;
  MAPIValueBuf: array[0..8] of Char;
  rType: Longint;
begin
  if not MAPICheckedW then
  begin
    MAPICheckedW := True;
    MAPIModuleW := 0;
    OSVersionInfo.dwOSVersionInfoSize := SizeOf(OSVersionInfo);
    GetVersionEx(OSVersionInfo);
    if (OSVersionInfo.dwMajorVersion > 3) or
      ((OSVersionInfo.dwMajorVersion = 3) and
       (OSVersionInfo.dwMinorVersion > 51)) then
    begin
      MAPIValueSize := sizeof(MAPIValueBuf);
      if RegOpenKeyEx(HKEY_LOCAL_MACHINE, 'SOFTWARE\Microsoft\Windows Messaging Subsystem',
        0, KEY_READ, hkWMS) <> ERROR_SUCCESS then Exit;
      if RegQueryValueEx(hkWMS, 'MAPI', nil, @rType, @MAPIValueBuf,
        @MAPIValueSize) <> ERROR_SUCCESS then Exit;
      RegCloseKey(hkWMS);
      if not ((MAPIValueBuf[0] = '1') and (MAPIValueBuf[1] = #0)) then Exit;
    end
    else if GetProfileInt('Mail', 'MAPI', 0) = 0 then Exit;
{$IFDEF MSWINDOWS}
    MAPIModuleW := LoadLibrary(PChar(MAPIDLL));
{$ENDIF}
  end;
end;

function CheckMapiLibrary: boolean;
begin
  InitMapi;
  Result:=MAPIModuleW<>0;
end;

function CheckMapiUnicode: boolean;
begin
  InitMapi;
  if @SendMailW = nil then
    @SendMailW := GetProcAddress(MAPIModuleW, 'MAPISendMailW');
  Result:=@SendMailW <> nil;
end;

function MapiSendMailW(lhSession: LHANDLE; ulUIParam: Cardinal;
  var lpMessageW: TMapiMessageW; flFlags: FLAGS; ulReserved: Cardinal): Cardinal;
begin
  InitMapi;
  if @SendMailW = nil then
    @SendMailW := GetProcAddress(MAPIModuleW, 'MAPISendMailW');
  if @SendMailW <> nil then
    Result := SendMailW(lhSession, ulUIParam, lpMessageW, flFlags, ulReserved)
  else Result := 1;
end;

initialization
finalization
  if MAPIModuleW <> 0 then FreeLibrary(MAPIModuleW);
end.
