unit rGroupBox_DTEdit;

{<
  PropSave_DTEdit is design time dialog used by PropSaveMain component.

  Updated version compatible with Delphi2010+.
  Full functional demo.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(May 2006)
  @lastmod(Dec 2013)
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, rImprovedComps, CheckLst, PropSaveMain;

type
  { design time dialog used by PropSaveMain component }
  TrGroupBox_DTEditForm = class(TForm)
    pc: TPageControl;
    ts1: TTabSheet;
    gbEmpty: TrGroupBox;
    btRefPos: TButton;
    btClose: TButton;
    gbRight: TrGroupBox;
    Label2: TLabel;
    lbAutoPos: TListBox;
    btUp: TButton;
    btDown: TButton;
    btDel1: TButton;
    btDelAll: TButton;
    btDelNoField: TButton;
    gbLeft: TrGroupBox;
    Label1: TLabel;
    lbControls: TListBox;
    btMove1: TButton;
    btMoveAll: TButton;
    gbProps: TrGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    cbVertAlign: TComboBox;
    eCol1: TrEditNum;
    eCol2: TrEditNum;
    eResW: TrEditNum;
    cbHorAlign: TComboBox;
    eResH: TrEditNum;
    PSFile: TPropSaveFile;
    PSMain: TPropSaveMain;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btCloseClick(Sender: TObject);
    procedure btMove1Click(Sender: TObject);
    procedure btMoveAllClick(Sender: TObject);
    procedure btDel1Click(Sender: TObject);
    procedure btDelAllClick(Sender: TObject);
    procedure btUpClick(Sender: TObject);
    procedure btDownClick(Sender: TObject);
    procedure lbAutoPosClick(Sender: TObject);
    procedure lbAutoPosKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure lbAutoPosDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure btDelNoFieldClick(Sender: TObject);
    procedure cbVertAlignChange(Sender: TObject);
    procedure btRefPosClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    GB: TrGroupBox;
    AutoResizeChild: TAutoResizeChild;
    FAutoPos: TStrings;
    procedure LoadProps;
    procedure SetProps;
    procedure ShowControls;
    procedure MoveI(List: TListBox; old, new: Integer);
  public
    { Public declarations }
  end;

function ShowrGroupBoxEditor(Component: TComponent): Boolean;

implementation

{$R *.dfm}
uses TypInfo, RosiCompGlobalCfg, rtool, rdlg;

function ShowrGroupBoxEditor(Component: TComponent): Boolean;
var
  F: TrGroupBox_DTEditForm;
begin
  Result:=true;
  F:=TrGroupBox_DTEditForm.Create(Application);
  try
    F.Caption:=Component.Name;
    if Component is TrGroupBox then
    begin
      F.GB:=Component as TrGroupBox;
      F.AutoResizeChild:=GetObjectProp(Component, 'AutoResizeChild') as TAutoResizeChild;
      F.FAutoPos:=GetObjectProp(F.AutoResizeChild, 'ChildCompsPos') as TStrings;
    end
    else
      raise Exception.Create('Unknown component type.');
    F.ShowModal;
    F.GB.RefreshAutoPosition;
  finally
    F.Free;
  end;
end;

procedure TrGroupBox_DTEditForm.FormCreate(Sender: TObject);
begin
  PSFile.LoadFile(GetRosiCompIniFile);
end;

procedure TrGroupBox_DTEditForm.FormShow(Sender: TObject);
begin
  pc.ActivePageIndex:=0;
  LoadProps;
  if lbAutoPos.Items.Count > 0 then
    lbAutoPos.ItemIndex:=0;
  lbAutoPosClick(nil);
end;

procedure TrGroupBox_DTEditForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SetProps;
end;

procedure TrGroupBox_DTEditForm.FormResize(Sender: TObject);
begin
  gbLeft.Width:=ts1.ClientWidth div 2 - 18;
end;

procedure TrGroupBox_DTEditForm.LoadProps;
begin
  lbAutoPos.Items.Assign(FAutoPos);
  ShowControls;
end;

procedure TrGroupBox_DTEditForm.SetProps;
begin
  FAutoPos.Assign(lbAutoPos.Items);
end;

procedure TrGroupBox_DTEditForm.btCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TrGroupBox_DTEditForm.ShowControls;
var
  i, j, tab, minidx: Integer;
  C: TControl;
begin
  lbControls.Items.Clear;
  for i:=0 to GB.ControlCount - 1 do
  begin
    C:=GB.Controls[i];
    if C is TWinControl then
      tab:=(C as TWinControl).TabOrder
    else
      tab:=999;
    if (C.Name <> '') and not(csSubComponent in C.ComponentStyle) then
      lbControls.Items.AddObject(C.Name, TObject(tab));
  end;
  // sort according tab
  for i:=0 to lbControls.Items.Count - 1 do
  begin
    minidx:=i;
    for j:=i + 1 to lbControls.Items.Count - 1 do
      if Integer(lbControls.Items.Objects[j]) < Integer(lbControls.Items.Objects[minidx]) then
        minidx:=j;
    lbControls.Items.Move(minidx, i);
  end;
end;

procedure TrGroupBox_DTEditForm.MoveI(List: TListBox; old, new: Integer);
begin
  if not MoveListBox(List, old, new) then
    Exit;
  lbAutoPosClick(nil);
end;

procedure TrGroupBox_DTEditForm.btMove1Click(Sender: TObject);
var
  s: string;
  i: Integer;
begin
  for i:=0 to lbControls.Items.Count - 1 do
    if lbControls.Selected[i] then
    begin
      s:=lbControls.Items[i];
      if lbAutoPos.Items.IndexOfName(s) = -1 then
      begin
        lbAutoPos.Items.Add(s + '=C;1;1;0;0;0');
        lbAutoPos.ItemIndex:=lbAutoPos.Items.Count - 1;
      end;
    end;
  lbAutoPosClick(nil);
end;

procedure TrGroupBox_DTEditForm.btMoveAllClick(Sender: TObject);
var
  i: Integer;
begin
  // if DlgQ('Add all columns?')<>mrYes then Exit;
  for i:=0 to lbControls.Items.Count - 1 do
    lbControls.Selected[i]:=true;
  btMove1.Click;
end;

procedure TrGroupBox_DTEditForm.btRefPosClick(Sender: TObject);
begin
  self.SetProps;
  GB.RefreshAutoPosition;
end;

procedure TrGroupBox_DTEditForm.btDel1Click(Sender: TObject);
begin
  lbAutoPos.DeleteSelected;
  lbAutoPosClick(nil);
end;

procedure TrGroupBox_DTEditForm.btDelAllClick(Sender: TObject);
begin
  if DlgQ('Delete all settings?') <> mrYes then
    Exit;
  lbAutoPos.Items.Clear;
  gbProps.Enabled:=false;
end;

procedure TrGroupBox_DTEditForm.btDelNoFieldClick(Sender: TObject);
var
  i: Integer;
begin
  // if DlgQ('Delete all orphan components (not existing field)?')<>mrYes then Exit;
  for i:=lbAutoPos.Items.Count - 1 downto 0 do
    if lbControls.Items.IndexOf(lbAutoPos.Items.Names[i]) < 0 then
    begin
      lbAutoPos.Items.Delete(i);
    end;
  lbAutoPosClick(nil);
end;

procedure TrGroupBox_DTEditForm.btUpClick(Sender: TObject);
begin
  MoveI(lbAutoPos, lbAutoPos.ItemIndex, lbAutoPos.ItemIndex - 1);
end;

procedure TrGroupBox_DTEditForm.btDownClick(Sender: TObject);
begin
  MoveI(lbAutoPos, lbAutoPos.ItemIndex, lbAutoPos.ItemIndex + 1);
end;

procedure TrGroupBox_DTEditForm.lbAutoPosDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept:=true;
  MoveI(lbAutoPos, lbAutoPos.ItemIndex, Y div lbAutoPos.ItemHeight + lbAutoPos.TopIndex);
end;

procedure TrGroupBox_DTEditForm.lbAutoPosKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_UP) and (ssCtrl in Shift) then
  begin
    MoveI(lbAutoPos, lbAutoPos.ItemIndex, lbAutoPos.ItemIndex - 1);
    Key:=0;
  end;
  if (Key = VK_DOWN) and (ssCtrl in Shift) then
  begin
    MoveI(lbAutoPos, lbAutoPos.ItemIndex, lbAutoPos.ItemIndex + 1);
    Key:=0;
  end;
  if (Key = VK_DELETE) then
  begin
    btDel1.Click;
    Key:=0;
  end;
end;

procedure TrGroupBox_DTEditForm.lbAutoPosClick(Sender: TObject);
var
  i, col1, col2, resw, resh: Integer;
  props, vertype, hortype, v: string;
begin
  i:=lbAutoPos.ItemIndex;
  gbProps.Enabled:=i >= 0;
  if i < 0 then
    Exit;

  props:=lbAutoPos.Items.ValueFromIndex[i];
  vertype:=rtool.FindStrPart(props, rGroupBoxDefSeparator, 1, true);
  col1:=StrToIntDef(rtool.FindStrPart(props, rGroupBoxDefSeparator, 2, true), 0);
  col2:=StrToIntDef(rtool.FindStrPart(props, rGroupBoxDefSeparator, 3, true), 0);
  resw:=StrToIntDef(rtool.FindStrPart(props, rGroupBoxDefSeparator, 4, true), 0);
  hortype:=rtool.FindStrPart(props, rGroupBoxDefSeparator, 5, true);
  resh:=StrToIntDef(rtool.FindStrPart(props, rGroupBoxDefSeparator, 6, true), 0);

  cbVertAlign.ItemIndex:=-1;
  for i:=0 to cbVertAlign.Items.Count - 1 do
  begin
    v:=Trim(rtool.FindStrPart(cbVertAlign.Items[i], '-', 1, true));
    if SameText(vertype,v) then
      cbVertAlign.ItemIndex:=i;
  end;
  cbHorAlign.ItemIndex:=0;
  for i:=0 to cbHorAlign.Items.Count - 1 do
    if Pos(hortype, cbHorAlign.Items[i]) = 1 then
      cbHorAlign.ItemIndex:=i;

  eCol1.AsInteger:=col1;
  eCol2.AsInteger:=col2;
  eResW.AsInteger:=resw;
  eResH.AsInteger:=resh;
end;

procedure TrGroupBox_DTEditForm.cbVertAlignChange(Sender: TObject);
var
  props, v: string;
  i: Integer;
begin
  i:=lbAutoPos.ItemIndex;
  if i < 0 then
    Exit;

  if eCol1.AsInteger > eCol2.AsInteger then
    eCol2.AsInteger:=eCol1.AsInteger;
  v:=Trim(rtool.FindStrPart(cbVertAlign.Items[cbVertAlign.ItemIndex], '-', 1, true));
  props:=v + rGroupBoxDefSeparator + eCol1.Text + rGroupBoxDefSeparator + eCol2.Text +
    rGroupBoxDefSeparator + eResW.Text + rGroupBoxDefSeparator + copy(cbHorAlign.Items[cbHorAlign.ItemIndex], 1, 1)
    + rGroupBoxDefSeparator + eResH.Text + rGroupBoxDefSeparator;
  lbAutoPos.Items.ValueFromIndex[i]:=props;
end;

end.
