unit dm_rdbaction;

{<
  rDBAction provides set of universal DB actions and menus.

  Full functional demo
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Jan 2008)
  @lastmod(Sep 2015)
}

interface

uses
  Windows, SysUtils, Classes, Menus, Forms, Controls, StdCtrls, Buttons,
  ActnList, ImgList, DB, DBGrids, rDBGrid, ComCtrls;

type
  TGetOtherActionState = procedure(Sender: TObject; rDBGrid: TrDBGrid; var Enabled: boolean) of object;
  TRunOtherAction = procedure(Sender: TObject; rDBGrid: TrDBGrid) of object;

{ Enabling and showing of Actions depends on TAG value of Active Grid,
  on UserRights value and on TAG value of each action

  TAG value of Actions:
  format: xyzz
  x - user rights for non edit actions
  y - user rights for edit actions
  zz - bitmask for disable according Dataset state

  UserRights value (00 - 99):
  format: xy
  x - user rights for non edit actions
  y - user rights for edit actions
  disable action if  user rights < part of Tag value of Action
  typical vaules:
  1: TOOL actions
  2: TOOL and ADD record actions
  3: enable TOOL, ADD and EDIT record actions
  4: enable TOOL, ADD, EDIT and DELETE Actions

  Bitmask for disable each action according Dataset state:
  1: Inactive
  2: ReadOnly
  4: IsEmpty
  8: Browse
  16: Edit
  32: Insert


  typical values of Action and states for disabling:
  ADD record: 0251 (Insert+Edit+ReadOnly+Inactive)
  EDIT record: 0355 (Insert+Edit+ReadOnly+IsEmpty+Inactive)
  DELETE record: 0455 (Insert+Edit+ReadOnly+IsEmpty+Inactive)
  TOOL Button: 1049  (Insert+Edit+Inactive)
  SAVE or CANCEL record: 0315  (Browse+IsEmpty+ReadOnly+Inactive)
  EXIT Button: 0048  (Insert+Edit)


  TAG value of Grid:
  (bitmask for hiding Actions according TAG value of Grid)
  -1: disable all Actions
  1: hide whole Edit menu
  2: hide whole Sort menu
  4: hide whole Filter menu
  8: hide whole Columns menu
  16: hide whole Props menu
  32: dummy
  64: dummy
  128: hide action acoClose
  256: hide action aceDuplic
  512: hide action aceSaveAdd
  1024: hide action aceAdd
  2048: hide action aceEdit
  4096: hide action aceDelete
  8192: hide action aceAutoEdit


  typical values of TAG value of Grid:
  all actions: 0
  not editing: 1 or 3841
  only Edit: 3840

}
  TrDBAction = class(TDataModule)
    aceAdd: TAction;
    aceDuplic: TAction;
    aceEdit: TAction;
    aceDelete: TAction;
    aceSave: TAction;
    aceSaveAdd: TAction;
    aceCancel: TAction;
    acoFind: TAction;
    acfFilterSel: TAction;
    acfFilterSelAnd: TAction;
    acfFilterSelOr: TAction;
    acfFilterForm: TAction;
    acfFilter: TAction;
    accColumns: TAction;
    acsSort1: TAction;
    acsSort2: TAction;
    acoExport: TAction;
    acoPrint: TAction;
    acoProperties: TAction;
    acoRefresh: TAction;
    accWidthAuto: TAction;
    accWidthSame: TAction;
    accWidthDefault: TAction;
    acsSortNone: TAction;
    acfFilterSave: TAction;
    acfFilterLoad: TAction;
    aceAutoEdit: TAction;
    acoClose: TAction;
    pmDBAction: TPopupMenu;
    mnEdit1: TMenuItem;
    mnRecNew1: TMenuItem;
    mnRecDuplic1: TMenuItem;
    mnRecEdit1: TMenuItem;
    mnRecDelete1: TMenuItem;
    mnEditN2: TMenuItem;
    mnRecSave1: TMenuItem;
    mnRecSaveNew1: TMenuItem;
    mnRecCancel1: TMenuItem;
    mnEditN3: TMenuItem;
    mnRecAutoEdit1: TMenuItem;
    mnN1: TMenuItem;
    mnFind1: TMenuItem;
    mnSort1: TMenuItem;
    mnSortAsc1: TMenuItem;
    mnSortDesc1: TMenuItem;
    mnSortN1: TMenuItem;
    mnSortNone1: TMenuItem;
    mnFilter1: TMenuItem;
    mnFilterSel1: TMenuItem;
    mnFilterAnd1: TMenuItem;
    mnFilterOr1: TMenuItem;
    mnFilterForm1: TMenuItem;
    mnFilterN1: TMenuItem;
    mnFilterOnOff1: TMenuItem;
    mnFilterN2: TMenuItem;
    mnFilterSave1: TMenuItem;
    mnFilterLoad1: TMenuItem;
    mnN2: TMenuItem;
    mnColumns1: TMenuItem;
    mnColumnsProps1: TMenuItem;
    mnColN1: TMenuItem;
    mnColWidthAuto1: TMenuItem;
    mnColWidthSame1: TMenuItem;
    mnColWidthDefault1: TMenuItem;
    mnProperties1: TMenuItem;
    mnN3: TMenuItem;
    mnRefresh1: TMenuItem;
    pmFilterSel: TPopupMenu;
    mnFilterSelAnd2: TMenuItem;
    mnFilterSelOr2: TMenuItem;
    pmFilterList: TPopupMenu;
    mnFilterSave2: TMenuItem;
    mnFilterLoad2: TMenuItem;
    pmColumnWidth: TPopupMenu;
    mnColWidthAuto2: TMenuItem;
    mnColWidthSame2: TMenuItem;
    mnColWidthDefault2: TMenuItem;
    acsSort3: TAction;
    accWidthData: TAction;
    mnColWidthData2: TMenuItem;
    mnColN3: TMenuItem;
    mnColN2: TMenuItem;
    mnColWidthData1: TMenuItem;
    mnSortNextCol1: TMenuItem;
    ActList: TActionList;
    accDefaultGrid: TAction;
    acoOtherAction: TAction;
    acoImport: TAction;
    acsSortDlg: TAction;
    mnSortDlg1: TMenuItem;
    pmSortList: TPopupMenu;
    mnSortTemp2: TMenuItem;
    mnFilterTemp2: TMenuItem;
    mnEditN1: TMenuItem;
    aceView: TAction;
    acoColor: TAction;
    mnExportData1: TMenuItem;
    mnExport1: TMenuItem;
    mnPrint1: TMenuItem;
    acoBatchEdit: TAction;
    mnBatchUpdate1: TMenuItem;
    mnImport1: TMenuItem;
    mnExportN1: TMenuItem;
    mnCopyData1: TMenuItem;
    mnCopyTitle1: TMenuItem;
    acoCopyRow: TAction;
    acoCopyCol: TAction;
    acoCopyAll: TAction;
    acoCopyIncTitle: TAction;
    mnCopyCol1: TMenuItem;
    mnCopyRec1: TMenuItem;
    mnCopyTbl1: TMenuItem;
    mnCopyN1: TMenuItem;
    acoCopyCell: TAction;
    pmCopyData: TPopupMenu;
    mnCopyCol2: TMenuItem;
    mnCopyRec2: TMenuItem;
    mnCopyTbl2: TMenuItem;
    mnCopyN2: TMenuItem;
    mnCopyTitle2: TMenuItem;
    acoPrintDirect: TAction;
    mnPrintDirect1: TMenuItem;
    acfFilterDlg: TAction;
    mnFilterDlg1: TMenuItem;
    mnFilterN3: TMenuItem;
    acoPaste: TAction;
    mnPaste: TMenuItem;
    acoCopyIncHidden: TAction;
    mnCopyHidden1: TMenuItem;
    mnCopyHidden2: TMenuItem;
    ImgEna: TImageList;
    ImgDis: TImageList;
    procedure DataModuleCreate(Sender: TObject);
    procedure aceAddExecute(Sender: TObject);
    procedure accColumnsExecute(Sender: TObject);
    procedure accWidthAutoExecute(Sender: TObject);
    procedure accWidthSameExecute(Sender: TObject);
    procedure accWidthDefaultExecute(Sender: TObject);
    procedure acfFilterSelExecute(Sender: TObject);
    procedure acsSort1Execute(Sender: TObject);
    procedure acsSortNoneExecute(Sender: TObject);
    procedure acoFindExecute(Sender: TObject);
    procedure acoExportExecute(Sender: TObject);
    procedure acoPropertiesExecute(Sender: TObject);
    procedure acoRefreshExecute(Sender: TObject);
    procedure acoCloseExecute(Sender: TObject);
    procedure accWidthDataExecute(Sender: TObject);
    procedure acdIfBrowseExecute(Sender: TObject);
    procedure acoPrintExecute(Sender: TObject);
    procedure ActListUpdate(Action: TBasicAction; var Handled: boolean);
    procedure acoOtherActionExecute(Sender: TObject);
    procedure acsSortDlgExecute(Sender: TObject);
    procedure mnSortTemp2Click(Sender: TObject);
    procedure pmSortListPopup(Sender: TObject);
    procedure mnFilterTempClick(Sender: TObject);
    procedure pmFilterListPopup(Sender: TObject);
    procedure acoColorExecute(Sender: TObject);
    procedure acoCopyRowExecute(Sender: TObject);
    procedure acoCopyColExecute(Sender: TObject);
    procedure acoCopyAllExecute(Sender: TObject);
    procedure acoCopyCellExecute(Sender: TObject);
    procedure acoPrintDirectExecute(Sender: TObject);
    procedure acoPasteExecute(Sender: TObject);
  private
    { Private declarations }
    G, StoredG: TrDBGrid;
    URights: integer;
    URightsEdit: integer;
    acShortCuts: array of integer;
    acEditSC: integer;
    function IsUserRightsOk(A: TAction): boolean;
  public
    { Public declarations }
    OnRunOtherAction: TRunOtherAction;
    OnGetOtherActionState: TGetOtherActionState;
    procedure SetActiveGrid(Grid: TrDBGrid);
    function GetActiveGrid: TrDBGrid;
    procedure SetUserRights(EditRights: boolean; actRights: integer);
    procedure AddDBToolMenu(M: TMenuItem);
    function CheckEditState(ShowDialog: boolean): boolean;
    procedure CheckShortcut(var Key: Word; Shift: TShiftState);
    procedure RefreshAction;
    procedure StoreActiveGrid;
    procedure RestoreActiveGrid;
    procedure RelinkActionTo(C: TWinControl; NewDBAction: TrDBAction);
    procedure OnSetDefaultGridColumns(Sender: TObject);
  end;

var
  rDBAction: TrDBAction;

implementation

{$R *.dfm}
uses RosiCompGlobalCfg, rstring, rdlg, rtool, rdbtool,
  rDBFind, rDBFilter, rDBFields, rDBExport,
  rDBGridProps, rDBFilterSaver, rDBPrint, rDBPrintDirect, rDBSort2,
  rDBGridCellParams;

const
  acCatMax = 4;
  acCatStr: array [0 .. acCatMax] of string = ('Edit', 'Sort', 'Filter', 'Column', 'Props');
  acCatMask: array [0 .. acCatMax] of integer = (1, 2, 4, 8, 16);

  acSpecMax = 6;
  acSpecStr: array [0 .. acSpecMax] of string = ('acoClose', 'aceDuplic', 'aceSaveAdd', 'aceAdd', 'aceEdit', 'aceDelete', 'aceAutoEdit');
  acSpecMask: array [0 .. acSpecMax] of integer = (128, 256, 512, 1024, 2048, 4096, 8192);

procedure TrDBAction.DataModuleCreate(Sender: TObject);
var
  i: integer;
begin
  G:=nil;
  RosiCompConfig.rDBFieldsCfg.OnSetDefaultGridColumns:=OnSetDefaultGridColumns;
  // save old shortcuts
  SetLength(acShortCuts, ActList.ActionCount);
  for i:=0 to ActList.ActionCount - 1 do
    acShortCuts[i]:=(ActList.Actions[i] as TAction).ShortCut;
  acEditSC:=aceEdit.ShortCut;
  URights:=9;
  URightsEdit:=9;
  // RefreshAction;
end;

procedure TrDBAction.ActListUpdate(Action: TBasicAction; var Handled: boolean);
begin
  RefreshAction;
  Handled:=true;
end;

procedure TrDBAction.SetActiveGrid(Grid: TrDBGrid);
begin
  G:=Grid;
  RefreshAction;
end;

function TrDBAction.GetActiveGrid: TrDBGrid;
begin
  Result:=G;
end;

procedure TrDBAction.SetUserRights(EditRights: boolean; actRights: integer);
begin
  if actRights in [0 .. 9] then
  begin
    if EditRights then
      URightsEdit:=actRights
    else
      URights:=actRights;
    RefreshAction;
  end
  else
    Exception.Create('Invalid user rights value.');
end;

function TrDBAction.IsUserRightsOk(A: TAction): boolean;
begin
  if A.Category = 'Edit' then
    Result:=((A.Tag div 100) mod 10) <= URightsEdit
  else
    Result:=((A.Tag div 1000) mod 10) <= URights;
end;

procedure TrDBAction.AddDBToolMenu(M: TMenuItem);
var
  A: integer;
  procedure CopyItems(miFrom, miTo: TMenuItem);
  var
    i: integer;
    newm: TMenuItem;
  begin
    newm:=TMenuItem.Create(miTo.Owner);
    newm.Caption:=miFrom.Caption;
    newm.Action:=miFrom.Action;
    for i:=0 to miFrom.Count - 1 do
      CopyItems(miFrom.Items[i], newm);
    miTo.Add(newm);
  end;

begin
  // Add Toolmenu to M
  for A:=2 to pmDBAction.Items.Count - 1 do
    CopyItems(pmDBAction.Items[A], M);
end;

function TrDBAction.CheckEditState(ShowDialog: boolean): boolean;
begin
  if IsEditing(G) and ShowDialog then
    try
      case DlgQCancel(sSaveChangesQ) of
        mrYes:
          aceSave.Execute;
        mrNo:
          aceCancel.Execute;
      end;
    except
      on E: Exception do
        if not(E is EAbort) then
          ShowErr(sSaveRecordErr, E, false);
    end;
  Result:=not IsEditing(G);
end;

procedure TrDBAction.CheckShortcut(var Key: Word; Shift: TShiftState);
var
  i: integer;
  A: TAction;
begin
  if Key = 0 then
    Exit;
  for i:=0 to ActList.ActionCount - 1 do
  begin
    A:=(ActList.Actions[i] as TAction);
    if A.ShortCut = ShortCut(Key, Shift) then
    begin
      Key:=0;
      A.Execute;
      Break;
    end;
  end;
  if (Key in [VK_ESCAPE]) then
  begin
    aceCancel.Execute;
    Key:=0;
  end;
end;

procedure TrDBAction.RefreshAction;
var
  i, j, tg, grmsk: integer;
  tgEna, mskEna, Vis, ena: boolean;
  A: TAction;
begin
  tg:=-1;
  if Assigned(G) then
  begin
    tg:=G.Tag;
    if G.ReadOnly then
      tg:=tg or 1;
  end;
  // set mask according grid and dataset state
  if tg = -1 then
    grmsk:=255
  else
  begin
    grmsk:=1;
    if Assigned(G.DataSource) and Assigned(G.DataSource.DataSet) then
      try
        case G.DataSource.DataSet.State of
          dsInsert:
            grmsk:=32;
          dsEdit:
            grmsk:=16;
          dsBrowse:
            grmsk:=8;
          else
            grmsk:=1;
          end;
        if G.DataSource.DataSet.Active then
        begin
          // if G.ReadOnly then grMsk:=1;
          if G.DataSource.DataSet.IsEmpty then
            grmsk:=grmsk or 4;
          // if not (dgEditing in G.Options) then grmsk:=grmsk or 2;
          if not(G.DataSource.DataSet.CanModify) then
            grmsk:=grmsk or 2;
        end;
      except
      end;
  end;

  for i:=0 to ActList.ActionCount - 1 do
  begin
    A:=(ActList.Actions[i] as TAction);
    // Set Action Enable according Grid Tag
    tgEna:=true;
    for j:=0 to acCatMax do
      if A.Category = acCatStr[j] then
        tgEna:=tg and acCatMask[j] = 0;
    // Set Action Enable according Dataset State and UserRights
    mskEna:=((A.Tag mod 100) and grmsk = 0) and IsUserRightsOk(A);
    // Set Refresh action according OnRefreshData procedure
    if A = acoRefresh then
      mskEna:=mskEna and Assigned(G) and (Assigned(G.Sorter) or Assigned(G.OnRefreshData));
    // Set Action Enable
    A.Enabled:=tgEna and mskEna;
    // Set special actions visible
    Vis:=true;
    if tg <> -1 then
      for j:=0 to acSpecMax do
        if A.Name = acSpecStr[j] then
          Vis:=tg and acSpecMask[j] = 0;
    A.Visible:=Vis;
    // Set shortcut
    if tgEna and Vis then
      A.ShortCut:=acShortCuts[i]
    else
      A.ShortCut:=0;

    if A.Tag = 0 then
    begin
      ena:=false;
      if Assigned(OnGetOtherActionState) then
        OnGetOtherActionState(A, G, ena);
      A.Enabled:=ena;
    end;
  end;

  // Set shortcut for Edit
  // F2 is used to start to edit record and later
  // in Grid to start to edit selected field,
  // so shortcut should be deleted if aceEdit is disable
  if aceEdit.Enabled then
    aceEdit.ShortCut:=acEditSC
  else
    aceEdit.ShortCut:=0;

  // Set Popup Menu visible
  mnEdit1.Visible:=tg and 1 = 0;
  mnN1.Visible:=mnEdit1.Visible;
  mnSort1.Visible:=tg and 2 = 0;
  mnFilter1.Visible:=tg and 4 = 0;
  mnColumns1.Visible:=tg and 8 = 0;

  // Set particular Actions
  { // Disable AutoEdit if Edit is not allowed
    if not IsUserRightsOk(aceEdit) then
    try
    // G.DataSource.AutoEdit:=false;
    except
    end;
    }
  // Set other spec Actions
  if Assigned(G) and Assigned(G.DataSource) and Assigned(G.DataSource.DataSet) then
    try
      // Cancel ADDed record if Add is not allowed
      if not aceAdd.Visible and G.DataSource.DataSet.Active and (G.DataSource.DataSet.State = dsInsert) then
        G.DataSource.DataSet.Cancel;

      accWidthAuto.Checked:=G.ColumnWidth = cwAutoWidth;
      accWidthSame.Checked:=G.ColumnWidth = cwSameWidth;
      aceAutoEdit.Checked:=G.DataSource.AutoEdit;
      acfFilter.Checked:=G.DataSource.DataSet.Filtered;
      acfFilterLoad.Enabled:=Assigned(RosiCompConfig.Global_PropSaveFile);
      acfFilterSave.Enabled:=acfFilterLoad.Enabled and acfFilter.Checked;
    except
    end;
  if RosiCompConfig.rDBGridPropsCfg.HideCellParams then
    self.acoColor.Visible:=false;
end;

procedure TrDBAction.aceAddExecute(Sender: TObject);
begin
  if Sender = aceAdd then
    AddRecordEx(G);
  if Sender = aceDuplic then
    DuplicRecord(G, 1, false);
  if Sender = aceEdit then
    G.DataSource.DataSet.Edit;
  if Sender = aceDelete then
    G.DataSource.DataSet.Delete;
  if Sender = aceSave then
    G.DataSource.DataSet.Post;
  if Sender = aceSaveAdd then
  begin
    G.DataSource.DataSet.Post;
    AddRecordEx(G);
  end;
  if Sender = aceCancel then
    G.DataSource.DataSet.Cancel;
  if Sender = aceAutoEdit then
    G.DataSource.AutoEdit:=not G.DataSource.AutoEdit;
end;

procedure TrDBAction.accColumnsExecute(Sender: TObject);
begin
  ChooseFields(G);
end;

procedure TrDBAction.accWidthAutoExecute(Sender: TObject);
begin
  if G.ColumnWidth = cwAutoWidth then
    G.ColumnWidth:=cwNone
  else
    G.ColumnWidth:=cwAutoWidth;
end;

procedure TrDBAction.accWidthSameExecute(Sender: TObject);
begin
  if G.ColumnWidth = cwSameWidth then
    G.ColumnWidth:=cwNone
  else
    G.ColumnWidth:=cwSameWidth;
end;

procedure TrDBAction.accWidthDataExecute(Sender: TObject);
begin
  G.ColumnWidth:=cwDataWidth;
  G.ColumnWidth:=cwNone;
end;

procedure TrDBAction.accWidthDefaultExecute(Sender: TObject);
begin
  G.ColumnWidth:=cwDefaultWidth;
  G.ColumnWidth:=cwNone;
end;

procedure TrDBAction.acfFilterSelExecute(Sender: TObject);
var ref: boolean;
begin
  ref:=false;
  if Sender = acfFilterSel then
    ref:=FilterBySel(G, fjNone);
  if Sender = acfFilterSelAnd then
    ref:=FilterBySel(G, fjAnd);
  if Sender = acfFilterSelOr then
    ref:=FilterBySel(G, fjOr);
  if Sender = acfFilterForm then
    ref:=FilterRec(G);
  if Sender = acfFilterDlg then
    G.ShowFilterDlg;
  if Sender = acfFilter then
    try
      if G.DataSource.DataSet.Filtered then
        SetFilterString(G.DataSource.DataSet, '', true)
      else
        SetFilterString(G.DataSource.DataSet, G.DataSource.DataSet.Filter, true);
      ref:=true;
    except
    end;
  if Sender = acfFilterSave then
    SaveFilter(G);
  if Sender = acfFilterLoad then
    ref:=LoadFilter(G);
  if ref then
    G.RefreshFilteredFieldNames;
end;

procedure TrDBAction.acsSort1Execute(Sender: TObject);
var
  fn: string;
begin
  try
    fn:=G.SelectedField.FieldName;
    if Sender = acsSort3 then
    begin
      if G.PosInSortField(fn) = 0 then
        G.ChangeSort(G.SortFieldName + ',' + fn, G.SortDesc)
    end
    else
      G.ChangeSort(G.SelectedField.FieldName, Sender = acsSort2);
  except
  end;
end;

procedure TrDBAction.acsSortDlgExecute(Sender: TObject);
begin
  SortRec(G);
end;

procedure TrDBAction.acsSortNoneExecute(Sender: TObject);
begin
  G.SortFieldName:='';
  G.ChangeSort('', false);
end;

procedure TrDBAction.acoFindExecute(Sender: TObject);
begin
  FindRec(G);
end;

procedure TrDBAction.acoExportExecute(Sender: TObject);
begin
  DBExport(G, acfFilter.Enabled, accColumns.Enabled);
end;

procedure TrDBAction.acoPropertiesExecute(Sender: TObject);
begin
  DBGridProperties(G, accColumns.Enabled, true, acfFilter.Enabled);
end;

procedure TrDBAction.acoRefreshExecute(Sender: TObject);
begin
  try
    if Assigned(G.Sorter) then
      G.Sorter.DoRefresh(G,G.DataSource.DataSet)
    else
      if Assigned(G.OnRefreshData) then
        G.OnRefreshData(G);
  except
  end;
end;

procedure TrDBAction.acoCloseExecute(Sender: TObject);
begin
  if CheckEditState(true) then
    Screen.ActiveForm.Close;
end;

procedure TrDBAction.acoColorExecute(Sender: TObject);
begin
  rDBGridCellParams.DBGridCellParams(G);
end;

procedure TrDBAction.acoCopyAllExecute(Sender: TObject);
begin
  G.CopyAllData(acoCopyIncTitle.Checked,acoCopyIncHidden.Checked);
end;

procedure TrDBAction.acoCopyCellExecute(Sender: TObject);
begin
  G.CopyCellData;
end;

procedure TrDBAction.acoCopyColExecute(Sender: TObject);
begin
  G.CopyActiveFieldData(acoCopyIncTitle.Checked);
end;

procedure TrDBAction.acoCopyRowExecute(Sender: TObject);
begin
  G.CopyActiveRecordData(acoCopyIncTitle.Checked,acoCopyIncHidden.Checked);
end;

procedure TrDBAction.acdIfBrowseExecute(Sender: TObject);
begin
  if Sender is TButton then (Sender as TButton)
    .OnClick(self);
  if Sender is TSpeedButton then (Sender as TSpeedButton)
    .OnClick(self);
  if Sender is TMenuItem then (Sender as TMenuItem)
    .OnClick(self);
end;

procedure TrDBAction.acoPasteExecute(Sender: TObject);
begin
  if Assigned(G) then
    G.PasteData(true);
end;

procedure TrDBAction.acoPrintDirectExecute(Sender: TObject);
begin
  DBPrintDirect(G, accColumns.Enabled, acfFilter.Enabled, '');
end;

procedure TrDBAction.acoPrintExecute(Sender: TObject);
begin
  DBPrint(G, accColumns.Enabled, acfFilter.Enabled, '');
end;

procedure TrDBAction.StoreActiveGrid;
begin
  StoredG:=G;
end;

procedure TrDBAction.RestoreActiveGrid;
begin
  SetActiveGrid(StoredG);
end;

procedure TrDBAction.RelinkActionTo(C: TWinControl; NewDBAction: TrDBAction);
var
  i, j: integer;
  aC: TControl;
begin
  for i:=0 to C.ControlCount - 1 do
  begin
    aC:=C.Controls[i];
    for j:=0 to ActList.ActionCount - 1 do
    begin
      if aC.Action = ActList.Actions[j] then
        aC.Action:=NewDBAction.ActList.Actions[j];
      Continue;
    end;
    if aC is TToolButton then
      with aC as TToolButton do
      begin
        if DropDownMenu = pmFilterSel then
          DropDownMenu:=NewDBAction.pmFilterSel;
        if DropDownMenu = pmFilterList then
          DropDownMenu:=NewDBAction.pmFilterList;
        if DropDownMenu = pmColumnWidth then
          DropDownMenu:=NewDBAction.pmColumnWidth;
        if DropDownMenu = pmSortList then
          DropDownMenu:=NewDBAction.pmSortList;
        if DropDownMenu = pmCopyData then
          DropDownMenu:=NewDBAction.pmCopyData;
      end;
    // the same for child controls
    if aC is TWinControl then
      RelinkActionTo(aC as TWinControl, NewDBAction);
  end;
end;

procedure TrDBAction.OnSetDefaultGridColumns(Sender: TObject);
begin
  acoOtherActionExecute(accDefaultGrid);
end;

procedure TrDBAction.acoOtherActionExecute(Sender: TObject);
begin
  if Assigned(OnRunOtherAction) then
    OnRunOtherAction(Sender, G);
end;

procedure TrDBAction.pmFilterListPopup(Sender: TObject);
var
  L: TStrings;
  i: integer;
begin
  while pmFilterList.Items.Count > 3 do
    pmFilterList.Items.Delete(3);
  L:=TStringList.Create;
  try
    rDBFilterSaver.LoadFilterNames(G, L);
    if L.Count = 0 then
    begin
      pmFilterList.Items.Add(rtool.CreateMenuItem(self, sNoSavedItems, -1, mnSortTemp2Click));
      pmFilterList.Items[3].Enabled:=false;
    end
    else
      for i:=0 to L.Count - 1 do
        pmFilterList.Items.Add(rtool.CreateMenuItem(self, L[i], integer(L.Objects[i]), mnFilterTempClick));
  finally
    L.Free;
  end;
end;

procedure TrDBAction.mnFilterTempClick(Sender: TObject);
begin
  if rDBFilterSaver.ActiveFilterName(G, (Sender as TMenuItem).Caption) then
    G.RefreshFilteredFieldNames;
end;

procedure TrDBAction.pmSortListPopup(Sender: TObject);
var
  L: TStrings;
  i: integer;
begin
  L:=TStringList.Create;
  try
    rDBSort2.SortRec_LoadSortNames(G, L);
    pmSortList.Items.Clear;
    if L.Count = 0 then
    begin
      pmSortList.Items.Add(rtool.CreateMenuItem(self, sNoSavedItems, -1, mnSortTemp2Click));
      pmSortList.Items[0].Enabled:=false;
    end
    else
      for i:=0 to L.Count - 1 do
        pmSortList.Items.Add(rtool.CreateMenuItem(self, L[i], integer(L.Objects[i]), mnSortTemp2Click));
  finally
    L.Free;
  end;
end;

procedure TrDBAction.mnSortTemp2Click(Sender: TObject);
var
  i: integer;
begin
  i:=(Sender as TMenuItem).Tag;
  rDBSort2.SortRec_ActiveSortIdx(G, i);
end;

end.
